//
//  YGViewController.m
//  Dynamics
//
//  Created by yagom on 13. 10. 8..
//  Copyright (c) 2013년 yagom. All rights reserved.
//

#import "YGViewController.h"

@interface YGViewController ()

// Dynamic Animator
@property (nonatomic, strong) UIDynamicAnimator *animator;

// 중력 행동
@property (nonatomic, strong) UIGravityBehavior *gravity;

// 영역
@property (nonatomic, strong) UICollisionBehavior *collision;

// 바로 직전에 생성된 블럭
@property (nonatomic, weak) UIView *lastBlock;

@end

@implementation YGViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // UIDynamicAnimator 객체를 생성합니다.
    // 해당 애니메이션이 동작할 뷰를 이 뷰컨트롤러의 뷰로 지정했습니다.
    self.animator = [[UIDynamicAnimator alloc] initWithReferenceView:self.view];
    
    // 중력 행동을 수행 할 UIGravityBehavior 객체를 만듭니다.
    self.gravity = [[UIGravityBehavior alloc] init];
    
    // 위에서 생성한 UIDynamicAnimator 객체에 애니메이션 행동으로 영역 행동을 추가합니다.
    [self.animator addBehavior:self.gravity];
    
    // UICollisionBehavior는 해당 아이템들이 소솔될 영역을 지정해 줄 수 있습니다.
    self.collision = [[UICollisionBehavior alloc] init];
    
    // 아이템들이 영역 밖으로 벗어나지 않도록 설정합니다. NO로 설정하면 아이템들이 영역 밖으로 벗어날 수 있습니다.
    [self.collision setTranslatesReferenceBoundsIntoBoundary:YES];
    
    // UIDynamicAnimator 객체에 애니메이션 행동으로 영역 행동을 추가합니다.
    [self.animator addBehavior:self.collision];
    
}

- (IBAction)clickMakeBlockButton:(id)sender
{
    // 새로 생성될 블럭의 X 좌표를 임의로 만들어 줍니다.
    NSInteger randX = rand() % 300;
    
    // 사이즈도 똑같으면 재미없으니 임의의 숫자로 생성해줍니다.
    NSInteger randSize = (rand() % 50) + 30.0f;
    
    // 새로운 사각형 블럭을 만들어줍니다. 임의의 X좌표, 임의의 크기를 갖게 됩니다.
    UIView *block = [[UIView alloc] initWithFrame:CGRectMake((CGFloat)randX, 30.0f, (CGFloat)randSize, (CGFloat)randSize)];
    
    // 블럭은 역시 검은색!
    [block setBackgroundColor:[UIColor blackColor]];
    
    // 뷰 위에 얹어줍니다.
    [self.view addSubview:block];
    
    // 블럭이 움직일 수 있는 영역을 설정해주기 위하여 UICollisionBehavior 객체의 아이템으로 설정해줍니다.
    [self.collision addItem:block];
    
    // 블럭에 중력 행동을 넣어주기 위해 중력행동 아이템으로 블럭을 추가해줍니다.
    [self.gravity addItem:block];

    // UIAttachmentBehavior를 init할 때 붙이려는 item이 nil이면 익셉션 에러가 발생하므로 먼저 확인이 필요합니다.
    if ( nil != self.lastBlock )
    {
        // UIAttachmentBehavior 객체를 생성합니다.
        // 지금 생성해서 붙이려는 block과 이전에 생성된 블럭끼리 서로 붙게 만들 요량입니다.
        UIAttachmentBehavior *attachment = [[UIAttachmentBehavior alloc] initWithItem:block attachedToItem:self.lastBlock];
        
        // damping 프로퍼티는 진동억제 수치를 뜻합니다. 값이 클수록 진동 효과가 억제됩니다. 0.0 ~ 1.0
        attachment.damping = 0.1f;
        
        // frequency는 진동 주기를 뜻합니다. 값이 클수록 세집니다. 0.0 ~ 1.0
        attachment.frequency = 0.8f;
        
        // length는 두 아이템간의 거리를 뜻합니다. 본래 자동으로 세팅이 되지만 수동으로 이렇게 설정해 줄 수 있습니다.
        // 저는 두 아이템을 딱 붙이고 싶어서 거리를 이렇게 설정해봤습니다.
        attachment.length = (randSize / 2.0f) + (self.lastBlock.frame.size.width / 2.0f);
        
        // 물론 애니메이터에 행동을 등록해주어야 동작하겠지요.
        [self.animator addBehavior:attachment];
    }
    
    // 마지막으로 생성된 블럭을 지금 생성된 블록으로 바꿔줍니다.
    self.lastBlock = block;
}

@end
